searchData={"items":[{"type":"behaviour","title":"DBConnection","doc":"A behaviour module for implementing efficient database connection\nclient processes, pools and transactions.\n\n`DBConnection` handles callbacks differently to most behaviours. Some\ncallbacks will be called in the calling process, with the state\ncopied to and from the calling process. This is useful when the data\nfor a request is large and means that a calling process can interact\nwith a socket directly.\n\nA side effect of this is that query handling can be written in a\nsimple blocking fashion, while the connection process itself will\nremain responsive to OTP messages and can enqueue and cancel queued\nrequests.\n\nIf a request or series of requests takes too long to handle in the\nclient process a timeout will trigger and the socket can be cleanly\ndisconnected by the connection process.\n\nIf a calling process waits too long to start its request it will\ntimeout and its request will be cancelled. This prevents requests\nbuilding up when the database can not keep up.\n\nIf no requests are received for an idle interval, the pool will\nping all stale connections which can then ping the database to keep\nthe connection alive.\n\nShould the connection be lost, attempts will be made to reconnect with\n(configurable) exponential random backoff to reconnect. All state is\nlost when a connection disconnects but the process is reused.\n\nThe `DBConnection.Query` protocol provide utility functions so that\nqueries can be encoded and decoded without blocking the connection or pool.","ref":"DBConnection.html"},{"type":"behaviour","title":"Connection pools - DBConnection","doc":"DBConnection connections support using different pools via the `:pool` option\npassed to `start_link/2`. The default pool is `DBConnection.ConnectionPool`.\nAnother supported pool that is commonly used for tests is `DBConnection.Ownership`.\n\nFor now, using *custom* pools is not supported since the API for pools is not\npublic.","ref":"DBConnection.html#module-connection-pools"},{"type":"behaviour","title":"Errors - DBConnection","doc":"Most functions in this module raise a `DBConnection.ConnectionError` exception\nwhen failing to check out a connection from the pool.","ref":"DBConnection.html#module-errors"},{"type":"macro","title":"DBConnection.__using__/1","doc":"Use `DBConnection` to set the behaviour.","ref":"DBConnection.html#__using__/1"},{"type":"function","title":"DBConnection.available_connection_options/0","doc":"Returns the names of all possible options that you can pass to most functions\nin this module.\n\nThis is mostly useful for library authors that base their library on top of\n`DBConnection`, since they can use the return value of this function to perform\nvalidation on options only passing down these options to DBConnection.\n\nSee also `t:connection_option/0`.","ref":"DBConnection.html#available_connection_options/0"},{"type":"function","title":"DBConnection.available_start_options/0","doc":"Returns the names of all possible options that you can pass to `start_link/2`.\n\nThis is mostly useful for library authors that base their library on top of\n`DBConnection`, since they can use the return value of this function to perform\nvalidation on options only passing down these options to DBConnection.\n\nSee also `t:start_option/0`.","ref":"DBConnection.html#available_start_options/0"},{"type":"callback","title":"DBConnection.checkout/1","doc":"Checkouts the state from the connection process. Return `{:ok, state}`\nto allow the checkout or `{:disconnect, exception, state}` to disconnect.\n\nThis callback is called immediately after the connection is established\nand the state is never effetively checked in again. That's because\nDBConnection keeps the connection state in an ETS table that is moved\nbetween the different clients checking out connections. There is no\n`checkin` callback. The state is only handed back to the connection\nprocess during pings and (re)connects.\n\nThis callback is called in the connection process.","ref":"DBConnection.html#c:checkout/1"},{"type":"function","title":"DBConnection.child_spec/2","doc":"Creates a supervisor child specification for a pool of connections.\n\nSee `start_link/2` for options.","ref":"DBConnection.html#child_spec/2"},{"type":"function","title":"DBConnection.close/3","doc":"Close a prepared query on a database connection and return `{:ok, result}` on\nsuccess or `{:error, exception}` on error.\n\nThis function should be used to free resources held by the connection\nprocess and/or the database server.","ref":"DBConnection.html#close/3"},{"type":"function","title":"Options - DBConnection.close/3","doc":"* `:queue` - Whether to block waiting in an internal queue for the\n  connection's state (boolean, default: `true`). See \"Queue config\" in\n  `start_link/2` docs\n  * `:timeout` - The maximum time that the caller is allowed to perform\n  this operation (default: `15_000`)\n  * `:deadline` - If set, overrides `:timeout` option and specifies absolute\n  monotonic time in milliseconds by which caller must perform operation.\n  See `System` module documentation for more information on monotonic time\n  (default: `nil`)\n  * `:log` - A function to log information about a call, either\n  a 1-arity fun, `{module, function, args}` with `t:DBConnection.LogEntry.t/0`\n  prepended to `args` or `nil`. See `DBConnection.LogEntry` (default: `nil`)\n\nThe pool and connection module may support other options. All options\nare passed to `c:handle_close/3`.\n\nSee `prepare/3`.","ref":"DBConnection.html#close/3-options"},{"type":"function","title":"DBConnection.close!/3","doc":"Close a prepared query on a database connection and return the result. Raises\nan exception on error.\n\nSee `close/3`.","ref":"DBConnection.html#close!/3"},{"type":"callback","title":"DBConnection.connect/1","doc":"Connect to the database. Return `{:ok, state}` on success or\n`{:error, exception}` on failure.\n\nIf an error is returned it will be logged and another\nconnection attempt will be made after a backoff interval.\n\nThis callback is called in the connection process.","ref":"DBConnection.html#c:connect/1"},{"type":"function","title":"DBConnection.connection_module/1","doc":"Returns connection module used by the given connection pool.\n\nWhen given a process that is not a connection pool, returns an `:error`.","ref":"DBConnection.html#connection_module/1"},{"type":"callback","title":"DBConnection.disconnect/2","doc":"Disconnect from the database. Return `:ok`.\n\nThis callback is called from the connection process. The first argument is\neither the exception from a `:disconnect` 3-tuple returned by a previous\ncallback or an exception generated by the connection process.\n\nIf the state is controlled by a client and it exits or times out while\nprocessing a request, the last known state will be sent and the exception\nwill be a `DBConnection.ConnectionError`.\n\nWhen the connection is stopped, this callback will be invoked from `terminate`.\nThe last known state will be sent and the exception will be a `DBConnection.ConnectionError`\ncontaining the reason for the exit. To have the same happen on unexpected\nshutdowns, you may trap exits from the `connect` callback.","ref":"DBConnection.html#c:disconnect/2"},{"type":"function","title":"DBConnection.disconnect_all/3","doc":"Forces all connections in the pool to disconnect within the given interval\nin milliseconds.\n\nOnce this function is called, the pool will disconnect all of its connections\nas they are checked in or as they are pinged. Checked in connections will be\nrandomly disconnected within the given time interval. Pinged connections are\nimmediately disconnected - as they are idle (according to `:idle_interval`).\n\nIf the connection has a backoff configured (which is the case by default),\ndisconnecting means an attempt at a new connection will be done immediately\nafter, without starting a new process for each connection. However, if backoff\nhas been disabled, the connection process will terminate. In such cases,\ndisconnecting all connections may cause the pool supervisor to restart\ndepending on the max_restarts/max_seconds configuration of the pool,\nso you will want to set those carefully.","ref":"DBConnection.html#disconnect_all/3"},{"type":"function","title":"DBConnection.execute/4","doc":"Execute a prepared query with a database connection and return\n`{:ok, query, result}` on success or `{:error, exception}` if there was an error.\n\nIf the query is not prepared on the connection an attempt may be made to\nprepare it and then execute again.\n\n#","ref":"DBConnection.html#execute/4"},{"type":"function","title":"Options - DBConnection.execute/4","doc":"* `:queue` - Whether to block waiting in an internal queue for the\n  connection's state (boolean, default: `true`). See \"Queue config\" in\n  `start_link/2` docs\n  * `:timeout` - The maximum time that the caller is allowed to perform\n  this operation (default: `15_000`)\n  * `:deadline` - If set, overrides `:timeout` option and specifies absolute\n  monotonic time in milliseconds by which caller must perform operation.\n  See `System` module documentation for more information on monotonic time\n  (default: `nil`)\n  * `:log` - A function to log information about a call, either\n  a 1-arity fun, `{module, function, args}` with `t:DBConnection.LogEntry.t/0`\n  prepended to `args` or `nil`. See `DBConnection.LogEntry` (default: `nil`)\n\nThe pool and connection module may support other options. All options\nare passed to `handle_execute/4`.\n\nSee `prepare/3`.","ref":"DBConnection.html#execute/4-options"},{"type":"function","title":"DBConnection.execute!/4","doc":"Execute a prepared query with a database connection and return the\nresult. Raises an exception on error.\n\nSee `execute/4`","ref":"DBConnection.html#execute!/4"},{"type":"callback","title":"DBConnection.handle_begin/2","doc":"Handle the beginning of a transaction.\n\nReturn `{:ok, result, state}`/`{:ok, query, result, state}` to continue,\n`{status, state}` to notify caller that the transaction can not begin due\nto the transaction status `status`, or `{:disconnect, exception, state}`\nto error and disconnect. If `{:ok, query, result, state}` is returned,\nthe query will be used to log the begin command. Otherwise, it will be\nlogged as `begin`.\n\nA callback implementation should only return `status` if it\ncan determine the database's transaction status without side effect.\n\nThis callback is called in the client process.","ref":"DBConnection.html#c:handle_begin/2"},{"type":"callback","title":"DBConnection.handle_close/3","doc":"Close a query prepared by `c:handle_prepare/3` with the database. Return\n`{:ok, result, state}` on success and to continue,\n`{:error, exception, state}` to return an error and continue, or\n`{:disconnect, exception, state}` to return an error and disconnect.\n\nThis callback is called in the client process.","ref":"DBConnection.html#c:handle_close/3"},{"type":"callback","title":"DBConnection.handle_commit/2","doc":"Handle committing a transaction. Return `{:ok, result, state}` on successfully\ncommitting transaction, `{status, state}` to notify caller that the\ntransaction can not commit due to the transaction status `status`,\nor `{:disconnect, exception, state}` to error and disconnect.\n\nA callback implementation should only return `status` if it\ncan determine the database's transaction status without side effect.\n\nThis callback is called in the client process.","ref":"DBConnection.html#c:handle_commit/2"},{"type":"callback","title":"DBConnection.handle_deallocate/4","doc":"Deallocate a cursor declared by `c:handle_declare/4` with the database. Return\n`{:ok, result, state}` on success and to continue,\n`{:error, exception, state}` to return an error and continue, or\n`{:disconnect, exception, state}` to return an error and disconnect.\n\nThis callback is called in the client process.","ref":"DBConnection.html#c:handle_deallocate/4"},{"type":"callback","title":"DBConnection.handle_declare/4","doc":"Declare a cursor using a query prepared by `c:handle_prepare/3`. Return\n`{:ok, query, cursor, state}` to return altered query `query` and cursor\n`cursor` for a stream and continue, `{:error, exception, state}` to return an\nerror and continue or `{:disconnect, exception, state}` to return an error\nand disconnect.\n\nThis callback is called in the client process.","ref":"DBConnection.html#c:handle_declare/4"},{"type":"callback","title":"DBConnection.handle_execute/4","doc":"Execute a query prepared by `c:handle_prepare/3`. Return\n`{:ok, query, result, state}` to return altered query `query` and result\n`result` and continue, `{:error, exception, state}` to return an error and\ncontinue or `{:disconnect, exception, state}` to return an error and\ndisconnect.\n\nThis callback is called in the client process.","ref":"DBConnection.html#c:handle_execute/4"},{"type":"callback","title":"DBConnection.handle_fetch/4","doc":"Fetch the next result from a cursor declared by `c:handle_declare/4`. Return\n`{:cont, result, state}` to return the result `result` and continue using\ncursor, `{:halt, result, state}` to return the result `result` and close the\ncursor, `{:error, exception, state}` to return an error and close the\ncursor, `{:disconnect, exception, state}` to return an error and disconnect.\n\nThis callback is called in the client process.","ref":"DBConnection.html#c:handle_fetch/4"},{"type":"callback","title":"DBConnection.handle_prepare/3","doc":"Prepare a query with the database. Return `{:ok, query, state}` where\n`query` is a query to pass to `execute/4` or `close/3`,\n`{:error, exception, state}` to return an error and continue or\n`{:disconnect, exception, state}` to return an error and disconnect.\n\nThis callback is intended for cases where the state of a connection is\nneeded to prepare a query and/or the query can be saved in the\ndatabase to call later.\n\nThis callback is called in the client process.","ref":"DBConnection.html#c:handle_prepare/3"},{"type":"callback","title":"DBConnection.handle_rollback/2","doc":"Handle rolling back a transaction. Return `{:ok, result, state}` on successfully\nrolling back transaction, `{status, state}` to notify caller that the\ntransaction can not rollback due to the transaction status `status` or\n`{:disconnect, exception, state}` to error and disconnect.\n\nA callback implementation should only return `status` if it\ncan determine the database' transaction status without side effect.\n\nThis callback is called in the client and connection process.","ref":"DBConnection.html#c:handle_rollback/2"},{"type":"callback","title":"DBConnection.handle_status/2","doc":"Handle getting the transaction status. Return `{:idle, state}` if outside a\ntransaction, `{:transaction, state}` if inside a transaction,\n`{:error, state}` if inside an aborted transaction, or\n`{:disconnect, exception, state}` to error and disconnect.\n\nIf the callback returns a `:disconnect` tuples then `status/2` will return\n`:error`.","ref":"DBConnection.html#c:handle_status/2"},{"type":"callback","title":"DBConnection.ping/1","doc":"Called when the connection has been idle for a period of time. Return\n`{:ok, state}` to continue or `{:disconnect, exception, state}` to\ndisconnect.\n\nThis callback is called if no callbacks have been called after the\nidle timeout and a client process is not using the state. The idle\ntimeout can be configured by the `:idle_interval` and `:idle_limit`\noptions. This function can be called whether the connection is checked\nin or checked out.\n\nThis callback is called in the connection process.","ref":"DBConnection.html#c:ping/1"},{"type":"function","title":"DBConnection.prepare/3","doc":"Prepare a query with a database connection for later execution.\n\nIt returns `{:ok, query}` on success or `{:error, exception}` if there was\nan error.\n\nThe returned `query` can then be passed to `execute/4` and/or `close/3`\n\n#","ref":"DBConnection.html#prepare/3"},{"type":"function","title":"Options - DBConnection.prepare/3","doc":"* `:queue` - Whether to block waiting in an internal queue for the\n  connection's state (boolean, default: `true`). See \"Queue config\" in\n  `start_link/2` docs\n  * `:timeout` - The maximum time that the caller is allowed to perform\n  this operation (default: `15_000`)\n  * `:deadline` - If set, overrides `:timeout` option and specifies absolute\n  monotonic time in milliseconds by which caller must perform operation.\n  See `System` module documentation for more information on monotonic time\n  (default: `nil`)\n  * `:log` - A function to log information about a call, either\n  a 1-arity fun, `{module, function, args}` with `t:DBConnection.LogEntry.t/0`\n  prepended to `args` or `nil`. See `DBConnection.LogEntry` (default: `nil`)\n\nThe pool and connection module may support other options. All options\nare passed to `c:handle_prepare/3`.\n\n#","ref":"DBConnection.html#prepare/3-options"},{"type":"function","title":"Example - DBConnection.prepare/3","doc":"DBConnection.transaction(pool, fn conn ->\n      query = %Query{statement: \"SELECT * FROM table\"}\n      query = DBConnection.prepare!(conn, query)\n      try do\n        DBConnection.execute!(conn, query, [])\n      after\n        DBConnection.close(conn, query)\n      end\n    end)","ref":"DBConnection.html#prepare/3-example"},{"type":"function","title":"DBConnection.prepare!/3","doc":"Prepare a query with a database connection and return the prepared\nquery. An exception is raised on error.\n\nSee `prepare/3`.","ref":"DBConnection.html#prepare!/3"},{"type":"function","title":"DBConnection.prepare_execute/4","doc":"Prepare a query and execute it with a database connection and return both the\nprepared query and the result, `{:ok, query, result}` on success or\n`{:error, exception}` if there was an error.\n\nThe returned `query` can be passed to `execute/4` and `close/3`.\n\n#","ref":"DBConnection.html#prepare_execute/4"},{"type":"function","title":"Options - DBConnection.prepare_execute/4","doc":"* `:queue` - Whether to block waiting in an internal queue for the\n  connection's state (boolean, default: `true`). See \"Queue config\" in\n  `start_link/2` docs\n  * `:timeout` - The maximum time that the caller is allowed to perform\n  this operation (default: `15_000`)\n  * `:deadline` - If set, overrides `:timeout` option and specifies absolute\n  monotonic time in milliseconds by which caller must perform operation.\n  See `System` module documentation for more information on monotonic time\n  (default: `nil`)\n  * `:log` - A function to log information about a call, either\n  a 1-arity fun, `{module, function, args}` with `t:DBConnection.LogEntry.t/0`\n  prepended to `args` or `nil`. See `DBConnection.LogEntry` (default: `nil`)\n\n#","ref":"DBConnection.html#prepare_execute/4-options"},{"type":"function","title":"Example - DBConnection.prepare_execute/4","doc":"query                = %Query{statement: \"SELECT id FROM table WHERE id=$1\"}\n    {:ok, query, result} = DBConnection.prepare_execute(conn, query, [1])\n    {:ok, result2}       = DBConnection.execute(conn, query, [2])\n    :ok                  = DBConnection.close(conn, query)","ref":"DBConnection.html#prepare_execute/4-example"},{"type":"function","title":"DBConnection.prepare_execute!/4","doc":"Prepare a query and execute it with a database connection and return both the\nprepared query and result. An exception is raised on error.\n\nSee `prepare_execute/4`.","ref":"DBConnection.html#prepare_execute!/4"},{"type":"function","title":"DBConnection.prepare_stream/4","doc":"Create a stream that will prepare a query, execute it and stream results\nusing a cursor.\n\n#","ref":"DBConnection.html#prepare_stream/4"},{"type":"function","title":"Options - DBConnection.prepare_stream/4","doc":"* `:queue` - Whether to block waiting in an internal queue for the\n  connection's state (boolean, default: `true`). See \"Queue config\" in\n  `start_link/2` docs\n  * `:timeout` - The maximum time that the caller is allowed to perform\n  this operation (default: `15_000`)\n  * `:deadline` - If set, overrides `:timeout` option and specifies absolute\n  monotonic time in milliseconds by which caller must perform operation.\n  See `System` module documentation for more information on monotonic time\n  (default: `nil`)\n  * `:log` - A function to log information about a call, either\n  a 1-arity fun, `{module, function, args}` with `t:DBConnection.LogEntry.t/0`\n  prepended to `args` or `nil`. See `DBConnection.LogEntry` (default: `nil`)\n\nThe pool and connection module may support other options. All options\nare passed to `c:handle_prepare/3`, `c:handle_close/3`, `c:handle_declare/4`,\nand `c:handle_deallocate/4`.\n\n#","ref":"DBConnection.html#prepare_stream/4-options"},{"type":"function","title":"Example - DBConnection.prepare_stream/4","doc":"{:ok, results} = DBConnection.transaction(conn, fn conn ->\n      query = %Query{statement: \"SELECT id FROM table\"}\n      stream = DBConnection.prepare_stream(conn, query, [])\n      Enum.to_list(stream)\n    end)","ref":"DBConnection.html#prepare_stream/4-example"},{"type":"function","title":"DBConnection.reduce/3","doc":"Reduces a previously built stream or prepared stream.","ref":"DBConnection.html#reduce/3"},{"type":"function","title":"DBConnection.rollback/2","doc":"Rollback a database transaction and release lock on connection.\n\nWhen inside of a `transaction/3` call does a non-local return, using a\n`throw/1` to cause the transaction to enter a failed state and the\n`transaction/3` call returns `{:error, reason}`. If `transaction/3` calls are\nnested the connection is marked as failed until the outermost transaction call\ndoes the database rollback.\n\n#","ref":"DBConnection.html#rollback/2"},{"type":"function","title":"Example - DBConnection.rollback/2","doc":"{:error, :oops} = DBConnection.transaction(pool, fun(conn) ->\n      DBConnection.rollback(conn, :oops)\n    end)","ref":"DBConnection.html#rollback/2-example"},{"type":"function","title":"DBConnection.run/3","doc":"Acquire a lock on a connection and run a series of requests on it.\n\nThe return value of this function is the return value of `fun`.\n\nTo use the locked connection call the request with the connection\nreference passed as the single argument to the `fun`. If the\nconnection disconnects all future calls using that connection\nreference will fail.\n\n`run/3` and `transaction/3` can be nested multiple times but a\n`transaction/3` call inside another `transaction/3` will be treated\nthe same as `run/3`.\n\n> ##","ref":"DBConnection.html#run/3"},{"type":"function","title":"Checkout failures {: .warning} - DBConnection.run/3","doc":">\n> If we cannot check out a connection from the pool, this function raises a\n> `DBConnection.ConnectionError` exception.\n> If you want to handle these cases, you should rescue\n> `DBConnection.ConnectionError` exceptions when using `run/3`.","ref":"DBConnection.html#run/3-checkout-failures-warning"},{"type":"function","title":"Options - DBConnection.run/3","doc":"* `:queue` - Whether to block waiting in an internal queue for the\n  connection's state (boolean, default: `true`). See \"Queue config\" in\n  `start_link/2` docs\n  * `:timeout` - The maximum time that the caller is allowed to perform\n  this operation (default: `15_000`)\n  * `:deadline` - If set, overrides `:timeout` option and specifies absolute\n  monotonic time in milliseconds by which caller must perform operation.\n  See `System` module documentation for more information on monotonic time\n  (default: `nil`)\n\nThe pool may support other options.","ref":"DBConnection.html#run/3-options"},{"type":"function","title":"Example - DBConnection.run/3","doc":"{:ok, res} = DBConnection.run(conn, fn conn ->\n      DBConnection.execute!(conn, query, [])\n    end)","ref":"DBConnection.html#run/3-example"},{"type":"function","title":"DBConnection.start_link/2","doc":"Starts and links to a database connection process.\n\nBy default the `DBConnection` starts a pool with a single connection.\nThe size of the pool can be increased with `:pool_size`. A separate\npool can be given with the `:pool` option.\n\n#","ref":"DBConnection.html#start_link/2"},{"type":"function","title":"Options - DBConnection.start_link/2","doc":"* `:backoff_min` - The minimum backoff interval (default: `1_000`)\n  * `:backoff_max` - The maximum backoff interval (default: `30_000`)\n  * `:backoff_type` - The backoff strategy, `:stop` for no backoff and\n  to stop, `:exp` for exponential, `:rand` for random and `:rand_exp` for\n  random exponential (default: `:rand_exp`)\n  * `:configure` - A function to run before every connect attempt to\n  dynamically configure the options, either a 1-arity fun,\n  `{module, function, args}` with options prepended to `args` or `nil` where\n  only returned options are passed to connect callback (default: `nil`). This\n  function is called *in the connection process*.\n  * `:after_connect` - A function to run on connect using `run/3`, either\n  a 1-arity fun, `{module, function, args}` with `t:DBConnection.t/0` prepended\n  to `args` or `nil` (default: `nil`)\n  * `:after_connect_timeout` - The maximum time allowed to perform\n  function specified by `:after_connect` option (default: `15_000`)\n  * `:connection_listeners` - A list of process destinations to send\n    notification messages whenever a connection is connected or disconnected.\n    See \"Connection listeners\" below\n  * `:name` - A name to register the started process (see the `:name` option\n    in `GenServer.start_link/3`)\n  * `:pool` - Chooses the pool to be started (default: `DBConnection.ConnectionPool`). See\n    [\"Connection pools\"](#module-connection-pools).\n  * `:pool_size` - Chooses the size of the pool (default: `1`)\n  * `:idle_interval` - Controls the frequency we check for idle connections\n    in the pool. We then notify each idle connection to ping the database.\n    In practice, the ping happens within `idle_interval <= ping < 2 * idle_interval`.\n    Defaults to 1000ms.\n  * `:idle_limit` - The number of connections to ping on each `:idle_interval`.\n    Defaults to the pool size (all connections).\n  * `:queue_target` and `:queue_interval` - See \"Queue config\" below\n  * `:max_restarts` and `:max_seconds` - Configures the `:max_restarts` and\n    `:max_seconds` for the connection pool supervisor (see the `Supervisor` docs).\n    Typically speaking the connection process doesn't terminate, except due to\n    faults in DBConnection. However, if backoff has been disabled, then they\n    also terminate whenever a connection is disconnected (for instance, due to\n    client or server errors)\n  * `:show_sensitive_data_on_connection_error` - By default, `DBConnection`\n    hides all information during connection errors to avoid leaking credentials\n    or other sensitive information. You can set this option if you wish to\n    see complete errors and stacktraces during connection errors\n\n#","ref":"DBConnection.html#start_link/2-options"},{"type":"function","title":"Example - DBConnection.start_link/2","doc":"{:ok, conn} = DBConnection.start_link(mod, [idle_interval: 5_000])","ref":"DBConnection.html#start_link/2-example"},{"type":"function","title":"Queue config - DBConnection.start_link/2","doc":"Handling requests is done through a queue. When DBConnection is\nstarted, there are two relevant options to control the queue:\n\n  * `:queue_target` in milliseconds, defaults to 50ms\n  * `:queue_interval` in milliseconds, defaults to 1000ms\n\nOur goal is to wait at most `:queue_target` for a connection.\nIf all connections checked out during a `:queue_interval` takes\nmore than `:queue_target`, then we double the `:queue_target`.\nIf checking out connections take longer than the new target,\nthen we start dropping messages.\n\nFor example, by default our target is 50ms. If all connections\ncheckouts take longer than 50ms for a whole second, we double\nthe target to 100ms and we start dropping messages if the\ntime to checkout goes above the new limit.\n\nThis allows us to better plan for overloads as we can refuse\nrequests before they are sent to the database, which would\notherwise increase the burden on the database, making the\noverload worse.","ref":"DBConnection.html#start_link/2-queue-config"},{"type":"function","title":"Connection listeners - DBConnection.start_link/2","doc":"The `:connection_listeners` option allows one or more processes to be notified\nwhenever a connection is connected or disconnected. A listener may be a remote\nor local PID, a locally registered name, or a tuple in the form of\n`{registered_name, node}` for a registered name at another node.\n\nEach listener process may receive the following messages where `pid`\nidentifies the connection process:\n\n  * `{:connected, pid}`\n  * `{:disconnected, pid}`\n\nIf the value of `:connection_listeners` is a tuple like `{listeners, term}`, then\nthe messages are these instead:\n\n  * `{:connected, pid, term}`\n  * `{:disconnected, pid, term}`\n\nNote the disconnected messages are not guaranteed to be delivered if the\n`pid` for connection crashes. So it is recommended to monitor the connected\n`pid` if you want to track all disconnections.\n\nHere is an example of a `:connection_listener` implementation:\n\n    defmodule DBConnectionListener do\n      use GenServer\n\n      def start_link(opts) do\n        GenServer.start_link(__MODULE__, [], opts)\n      end\n\n      def get_notifications(pid) do\n        GenServer.call(pid, :read_state)\n      end\n\n      @impl true\n      def init(stack) when is_list(stack) do\n        {:ok, stack}\n      end\n\n      @impl true\n      def handle_call(:read_state, _from, state) do\n        {:reply, state, state}\n      end\n\n      @impl true\n      def handle_info({:connected, _pid} = msg, state) do\n        {:noreply, [msg | state]}\n      end\n\n      @impl true\n      def handle_info({_other_states, _pid} = msg, state) do\n        {:noreply, [msg | state]}\n      end\n    end\n\nYou can then start it, pass the PID in the `connection_listeners`\noption on `DBConnection.start_link/2` and, when needed, can query the notifications:\n\n    {:ok, pid} = DBConnectionListener.start_link([])\n    {:ok, _conn} = DBConnection.start_link(SomeModule, [connection_listeners: [pid]])\n    notifications = DBConnectionListener.get_notifications(pid)\n\n#","ref":"DBConnection.html#start_link/2-connection-listeners"},{"type":"function","title":"Tagging messages - DBConnection.start_link/2","doc":"If you pass `{listeners, tag}` as an option, you can specify an arbitrary `tag` term that will\nbe sent alongside all `:connected`/`:disconnected` messages. This is useful if you\nwant to track information about the pool a connection belongs to or any other information.\n\nThis feature is available since v2.6.0. Before this version `:connection_listeners` only\naccepted a list of listener processes.","ref":"DBConnection.html#start_link/2-tagging-messages"},{"type":"function","title":"Telemetry - DBConnection.start_link/2","doc":"A `[:db_connection, :connection_error]` event is published whenever a\nconnection checkout receives a `%DBConnection.ConnectionError{}`.\n\nMeasurements:\n\n  * `:count` - A fixed-value measurement which always measures 1.\n\nMetadata\n\n  * `:error` - The `DBConnection.ConnectionError` struct which triggered the event.\n\n  * `:opts` - All options given to the pool operation","ref":"DBConnection.html#start_link/2-telemetry"},{"type":"function","title":"DBConnection.status/2","doc":"Return the transaction status of a connection.\n\nThe callback implementation should return the transaction status according to\nthe database, and not make assumptions based on client-side state.\n\nThis function will raise a `DBConnection.ConnectionError` when called inside a\ndeprecated `transaction/3`.\n\n#","ref":"DBConnection.html#status/2"},{"type":"function","title":"Options - DBConnection.status/2","doc":"See module documentation. The pool and connection module may support other\noptions. All options are passed to `c:handle_status/2`.\n\n#","ref":"DBConnection.html#status/2-options"},{"type":"function","title":"Example - DBConnection.status/2","doc":"# outside of the transaction, the status is `:idle`\n    DBConnection.status(conn) #=> :idle\n\n    DBConnection.transaction(conn, fn conn ->\n      DBConnection.status(conn) #=> :transaction\n\n      # run a query that will cause the transaction to rollback, e.g.\n      # uniqueness constraint violation\n      DBConnection.execute(conn, bad_query, [])\n\n      DBConnection.status(conn) #=> :error\n    end)\n\n    DBConnection.status(conn) #=> :idle","ref":"DBConnection.html#status/2-example"},{"type":"function","title":"DBConnection.stream/4","doc":"Create a stream that will execute a prepared query and stream results using a\ncursor.\n\n#","ref":"DBConnection.html#stream/4"},{"type":"function","title":"Options - DBConnection.stream/4","doc":"* `:queue` - Whether to block waiting in an internal queue for the\n  connection's state (boolean, default: `true`). See \"Queue config\" in\n  `start_link/2` docs\n  * `:timeout` - The maximum time that the caller is allowed to perform\n  this operation (default: `15_000`)\n  * `:deadline` - If set, overrides `:timeout` option and specifies absolute\n  monotonic time in milliseconds by which caller must perform operation.\n  See `System` module documentation for more information on monotonic time\n  (default: `nil`)\n  * `:log` - A function to log information about a call, either\n  a 1-arity fun, `{module, function, args}` with `t:DBConnection.LogEntry.t/0`\n  prepended to `args` or `nil`. See `DBConnection.LogEntry` (default: `nil`)\n\nThe pool and connection module may support other options. All options\nare passed to `c:handle_declare/4` and `c:handle_deallocate/4`.\n\n#","ref":"DBConnection.html#stream/4-options"},{"type":"function","title":"Example - DBConnection.stream/4","doc":"DBConnection.transaction(pool, fn conn ->\n      query = %Query{statement: \"SELECT id FROM table\"}\n      query = DBConnection.prepare!(conn, query)\n      try do\n        stream = DBConnection.stream(conn, query, [])\n        Enum.to_list(stream)\n      after\n        # Make sure query is closed!\n        DBConnection.close(conn, query)\n      end\n    end)","ref":"DBConnection.html#stream/4-example"},{"type":"function","title":"DBConnection.transaction/3","doc":"Acquire a lock on a connection and run a series of requests inside a\ntransaction. The result of the transaction fun is return inside an `:ok`\ntuple: `{:ok, result}`.\n\nTo use the locked connection call the request with the connection\nreference passed as the single argument to the `fun`. If the\nconnection disconnects all future calls using that connection\nreference will fail.\n\n`run/3` and `transaction/3` can be nested multiple times. If a transaction is\nrolled back or a nested transaction `fun` raises the transaction is marked as\nfailed. All calls except `run/3`, `transaction/3`, `rollback/2`, `close/3` and\n`close!/3` will raise an exception inside a failed transaction until the outer\ntransaction call returns. All `transaction/3` calls will return\n`{:error, :rollback}` if the transaction failed or connection closed and\n`rollback/2` is not called for that `transaction/3`.\n\n#","ref":"DBConnection.html#transaction/3"},{"type":"function","title":"Options - DBConnection.transaction/3","doc":"* `:queue` - Whether to block waiting in an internal queue for the\n  connection's state (boolean, default: `true`). See \"Queue config\" in\n  `start_link/2` docs\n  * `:timeout` - The maximum time that the caller is allowed to perform\n  this operation (default: `15_000`)\n  * `:deadline` - If set, overrides `:timeout` option and specifies absolute\n  monotonic time in milliseconds by which caller must perform operation.\n  See `System` module documentation for more information on monotonic time\n  (default: `nil`)\n  * `:log` - A function to log information about begin, commit and rollback\n  calls made as part of the transaction, either a 1-arity fun,\n  `{module, function, args}` with `t:DBConnection.LogEntry.t/0` prepended to\n  `args` or `nil`. See `DBConnection.LogEntry` (default: `nil`)\n\nThe pool and connection module may support other options. All options\nare passed to `c:handle_begin/2`, `c:handle_commit/2` and\n`c:handle_rollback/2`.\n\n#","ref":"DBConnection.html#transaction/3-options"},{"type":"function","title":"Example - DBConnection.transaction/3","doc":"{:ok, res} = DBConnection.transaction(conn, fn conn ->\n      DBConnection.execute!(conn, query, [])\n    end)","ref":"DBConnection.html#transaction/3-example"},{"type":"type","title":"DBConnection.conn/0","doc":"","ref":"DBConnection.html#t:conn/0"},{"type":"type","title":"DBConnection.connection_option/0","doc":"An option you can pass to DBConnection functions.","ref":"DBConnection.html#t:connection_option/0"},{"type":"type","title":"DBConnection.cursor/0","doc":"","ref":"DBConnection.html#t:cursor/0"},{"type":"type","title":"DBConnection.option/0","doc":"An option you can pass to DBConnection functions (*deprecated*).\n\n> ##","ref":"DBConnection.html#t:option/0"},{"type":"type","title":"Deprecated {: .warning} - DBConnection.option/0","doc":">\n> This option is deprecated since v2.6.0. Use `t:connection_option/0` instead.","ref":"DBConnection.html#t:option/0-deprecated-warning"},{"type":"type","title":"DBConnection.params/0","doc":"","ref":"DBConnection.html#t:params/0"},{"type":"type","title":"DBConnection.query/0","doc":"","ref":"DBConnection.html#t:query/0"},{"type":"type","title":"DBConnection.result/0","doc":"","ref":"DBConnection.html#t:result/0"},{"type":"type","title":"DBConnection.start_option/0","doc":"","ref":"DBConnection.html#t:start_option/0"},{"type":"type","title":"DBConnection.status/0","doc":"","ref":"DBConnection.html#t:status/0"},{"type":"type","title":"DBConnection.t/0","doc":"Run or transaction connection reference.","ref":"DBConnection.html#t:t/0"},{"type":"module","title":"DBConnection.ConnectionPool","doc":"The default connection pool.\n\nThe queueing algorithm is based on [CoDel](https://queue.acm.org/appendices/codel.html).\n\nYou're not supposed to call any functions on this pool directly, but only pass this\nas the value of the `:pool` option in functions such as `DBConnection.start_link/2`.","ref":"DBConnection.ConnectionPool.html"},{"type":"module","title":"DBConnection.LogEntry","doc":"Struct containing log entry information.\n\nSee `t:t/0` for information on the fields.","ref":"DBConnection.LogEntry.html"},{"type":"type","title":"DBConnection.LogEntry.t/0","doc":"Log entry information.\n\n  * `:call` - The `DBConnection` function called\n  * `:query` - The query used by the function\n  * `:params` - The params passed to the function (if any)\n  * `:result` - The result of the call\n  * `:pool_time` - The length of time awaiting a connection from the pool (if\n  the connection was not already checked out)\n  * `:connection_time` - The length of time using the connection (if a\n  connection was used)\n  * `:decode_time` - The length of time decoding the result (if decoded the\n  result using `DBConnection.Query.decode/3`)\n  * `:idle_time` - The amount of time the connection was idle before use\n\nAll times are in the native time units of the VM, see\n`System.monotonic_time/0`.","ref":"DBConnection.LogEntry.html#t:t/0"},{"type":"module","title":"DBConnection.Ownership","doc":"A DBConnection pool that requires explicit checkout and checkin\nas a mechanism to coordinate between processes.","ref":"DBConnection.Ownership.html"},{"type":"module","title":"Options - DBConnection.Ownership","doc":"* `:ownership_mode` - When mode is `:manual`, all connections must\n    be explicitly checked out before by using `ownership_checkout/2`.\n    Otherwise, mode is `:auto` and connections are checked out\n    implicitly. `{:shared, owner}` mode is also supported so\n    processes are allowed on demand. On all cases, checkins are\n    explicit via `ownership_checkin/2`. Defaults to `:auto`.\n  * `:ownership_timeout` - The maximum time (in milliseconds) that a process\n    is allowed to own a connection or `:infinity`, default `120_000`.\n    This timeout exists mostly for sanity checking purposes and can be increased\n    at will, since DBConnection automatically checks in connections whenever\n    there is a mode change.\n  * `:ownership_log` - The `Logger.level` to log ownership changes, or `nil`\n    not to log, default `nil`.\n\nThere are also two experimental options, `:post_checkout` and `:pre_checkin`\nwhich allows a developer to configure what happens when a connection is\nchecked out and checked in. Those options are meant to be used during tests,\nand have the following behaviour:\n\n  * `:post_checkout` - it must be an anonymous function that receives the\n    connection module, the connection state and it must return either\n    `{:ok, connection_module, connection_state}` or\n    `{:disconnect, err, connection_module, connection_state}`. This allows\n    the developer to change the connection module on post checkout. However,\n    in case of disconnects, the return `connection_module` must be the same\n    as the `connection_module` given. Defaults to simply returning the given\n    connection module and state.\n\n  * `:pre_checkin` - it must be an anonymous function that receives the\n    checkin reason (`:checkin`, `{:disconnect, err}` or `{:stop, err}`),\n    the connection module and the connection state returned by `post_checkout`.\n    It must return either `{:ok, connection_module, connection_state}` or\n    `{:disconnect, err, connection_module, connection_state}` where the connection\n    module is the module given to `:post_checkout` Defaults to simply returning\n    the given connection module and state.","ref":"DBConnection.Ownership.html#module-options"},{"type":"module","title":"Callers lookup - DBConnection.Ownership","doc":"When checking out, the ownership pool first looks if there is a connection\nassigned to the current process and then checks if there is a connection\nassigned to any of the processes listed under the `$callers` process\ndictionary entry. The `$callers` entry is set by default for tasks from\nElixir v1.8.\n\nYou can also pass the `:caller` option on checkout with a pid and that\npid will be looked up first, instead of `self()`, and then we fall back\nto `$callers`.","ref":"DBConnection.Ownership.html#module-callers-lookup"},{"type":"function","title":"DBConnection.Ownership.ownership_allow/4","doc":"Allows the process given by `allow` to use the connection checked out\nby `owner_or_allowed`.\n\nIt may return `:ok` if the connection is checked out.\n`{:already, :owner | :allowed}` if the `allow` process already\nhas a connection. `owner_or_allowed` may either be the owner or any\nother allowed process. Returns `:not_found` if the given process\ndoes not have any connection checked out.","ref":"DBConnection.Ownership.html#ownership_allow/4"},{"type":"function","title":"DBConnection.Ownership.ownership_checkin/2","doc":"Checks a connection back in.\n\nA connection can only be checked back in by its owner.","ref":"DBConnection.Ownership.html#ownership_checkin/2"},{"type":"function","title":"DBConnection.Ownership.ownership_checkout/2","doc":"Explicitly checks a connection out from the ownership manager.\n\nIt may return `:ok` if the connection is checked out.\n`{:already, :owner | :allowed}` if the caller process already\nhas a connection, `:error` if it could be not checked out or\nraise if there was an error.","ref":"DBConnection.Ownership.html#ownership_checkout/2"},{"type":"function","title":"DBConnection.Ownership.ownership_mode/3","doc":"Changes the ownership mode.\n\n`mode` may be `:auto`, `:manual` or `{:shared, owner}`.\n\nThe operation will always succeed when setting the mode to\n`:auto` or `:manual`. It may fail with reason `:not_owner`\nor `:not_found` when setting `{:shared, pid}` and the\ngiven pid does not own any connection. May return\n`:already_shared` if another process set the ownership\nmode to `{:shared, _}` and is still alive.","ref":"DBConnection.Ownership.html#ownership_mode/3"},{"type":"module","title":"DBConnection.PrepareStream","doc":"","ref":"DBConnection.PrepareStream.html"},{"type":"type","title":"DBConnection.PrepareStream.t/0","doc":"","ref":"DBConnection.PrepareStream.html#t:t/0"},{"type":"protocol","title":"DBConnection.Query","doc":"The `DBConnection.Query` protocol is responsible for preparing and\nencoding queries.\n\nAll `DBConnection.Query` functions are executed in the caller process which\nmeans it's safe to, for example, raise exceptions or do blocking calls as\nthey won't affect the connection process.","ref":"DBConnection.Query.html"},{"type":"function","title":"DBConnection.Query.decode/3","doc":"Decode a result using a query.\n\nThis function is called to decode a result after it is returned by a\nconnection callback module.\n\nSee `DBConnection.execute/3`.","ref":"DBConnection.Query.html#decode/3"},{"type":"function","title":"DBConnection.Query.describe/2","doc":"Describe a query.\n\nThis function is called to describe a query after it is prepared using a\nconnection callback module.\n\nSee `DBConnection.prepare/3`.","ref":"DBConnection.Query.html#describe/2"},{"type":"function","title":"DBConnection.Query.encode/3","doc":"Encode parameters using a query.\n\nThis function is called to encode a query before it is executed using a\nconnection callback module.\n\nIf this function raises `DBConnection.EncodeError`, then the query is\nprepared once again.\n\nSee `DBConnection.execute/3`.","ref":"DBConnection.Query.html#encode/3"},{"type":"function","title":"DBConnection.Query.parse/2","doc":"Parse a query.\n\nThis function is called to parse a query term before it is prepared using a\nconnection callback module.\n\nSee `DBConnection.prepare/3`.","ref":"DBConnection.Query.html#parse/2"},{"type":"type","title":"DBConnection.Query.t/0","doc":"All the types that implement this protocol.","ref":"DBConnection.Query.html#t:t/0"},{"type":"module","title":"DBConnection.Stream","doc":"","ref":"DBConnection.Stream.html"},{"type":"type","title":"DBConnection.Stream.t/0","doc":"","ref":"DBConnection.Stream.html#t:t/0"},{"type":"exception","title":"DBConnection.ConnectionError","doc":"A generic connection error exception.\n\nThe raised exception might include the reason which would be useful\nto programmatically determine what was causing the error.","ref":"DBConnection.ConnectionError.html"},{"type":"exception","title":"DBConnection.EncodeError","doc":"","ref":"DBConnection.EncodeError.html"},{"type":"exception","title":"DBConnection.OwnershipError","doc":"An exception for when errors with ownership occur.","ref":"DBConnection.OwnershipError.html"},{"type":"exception","title":"DBConnection.TransactionError","doc":"","ref":"DBConnection.TransactionError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v2.6.0 (2023-10-15) - Changelog","doc":"* Enhancements\n  * Call `disconnect` on terminate\n  * Allow `handle_begin` callbacks to return query for logging purposes\n  * Add `:connection_listeners_tag`\n  * Add `DBConnection.available_connection_options/0`\n  * Add `DBConnection.available_start_options/0`","ref":"changelog.html#v2-6-0-2023-10-15"},{"type":"extras","title":"v2.5.0 (2023-04-10) - Changelog","doc":"* Internal changes\n  * No longer depend on `connection`","ref":"changelog.html#v2-5-0-2023-04-10"},{"type":"extras","title":"v2.4.3 (2022-11-22) - Changelog","doc":"* Bug fixes\n  * Fix bug where `disconnect_all/2` interval would be disabled above 4294ms\n  * Add `:idle_limit` to limit the amount of disconnections on a ping","ref":"changelog.html#v2-4-3-2022-11-22"},{"type":"extras","title":"v2.4.2 (2022-03-03) - Changelog","doc":"* Enhancements\n  * Add `DBConnection.connection_module/1`","ref":"changelog.html#v2-4-2-2022-03-03"},{"type":"extras","title":"v2.4.1 (2021-10-14) - Changelog","doc":"* Enhancements\n  * Add `DBConnection.disconnect_all/2`","ref":"changelog.html#v2-4-1-2021-10-14"},{"type":"extras","title":"v2.4.0 (2021-04-02) - Changelog","doc":"* Enhancements\n  * Add telemetry events for connection errors\n  * Use `:rand` default algorithm\n  * Allow decentralized lookups on DBConnection.Ownership","ref":"changelog.html#v2-4-0-2021-04-02"},{"type":"extras","title":"v2.3.1 (2020-11-25) - Changelog","doc":"* Enhancements\n  * Add `:connection_listeners` to `DBConnection.start_link/2`\n  * Allow connection `~> 1.0`","ref":"changelog.html#v2-3-1-2020-11-25"},{"type":"extras","title":"v2.3.0 (2020-10-14) - Changelog","doc":"This release requires Elixir v1.7+.\n\n* Bug fixes\n  * Fix deprecation warnings related to the use of `System.stacktrace()`","ref":"changelog.html#v2-3-0-2020-10-14"},{"type":"extras","title":"v2.2.2 (2020-04-22) - Changelog","doc":"* Bug fixes\n  * Make sure all idle connections in the pool are pinged on each idle interval","ref":"changelog.html#v2-2-2-2020-04-22"},{"type":"extras","title":"v2.2.1 (2020-02-04) - Changelog","doc":"* Enhancements\n  * Remove warnings","ref":"changelog.html#v2-2-1-2020-02-04"},{"type":"extras","title":"v2.2.0 (2019-12-11) - Changelog","doc":"* Enhancements\n  * Add `:idle_time` to `DBConnection.LogEntry`\n  * Ping all stale connections on idle interval\n  * Add `crash_reason` to relevant Logger error reports\n  * Ping all stale connections on idle interval. One possible downside of this approach is that we may shut down all connections at once and if there is a request around this time, the response time will be higher. However, this is likely better than the current approach, where we ping only the first one, which means we can have a pool of stale connections. The current behaviour is the same as in v1.0","ref":"changelog.html#v2-2-0-2019-12-11"},{"type":"extras","title":"v2.1.1 (2019-07-17) - Changelog","doc":"* Enhancements\n  * Reduce severity in client exits to info\n  * Improve error message on redirect checkout\n\n* Bug fixes\n  * Make sure ownership timeout is respected on automatic checkouts","ref":"changelog.html#v2-1-1-2019-07-17"},{"type":"extras","title":"v2.1.0 (2019-06-07) - Changelog","doc":"* Enhancements\n  * Require Elixir v1.6+\n  * Include client stacktrace on check out timeouts","ref":"changelog.html#v2-1-0-2019-06-07"}],"content_type":"text/markdown"}